'''''''''''''''''''''''''''''''''''''''''''
''  NavigatorBoard
''
''  Written April 2008 by Chuck Rice
'''''''''''''''''''''''''''''''''''''''''''

CON

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000

'Signal Constants

  cOutput       = %1
  cInput        = %0
  cHigh         = %1
  cLow          = %0


 
'' Pin allocations

  cEnablePin     =  0
  cClockPin      =  1
  cDataPin       =  2
  cServoPin      =  5                                      
  cSonarTrigger  =  6                                      
  cSonarData     =  7

  cGpsPin        =  8                                    
 
  cRxPin         = 26                                   'ComBoard Communications Pins
  cTxPin         = 27

' GPS Module Commands 

cGetInfo         =     $00 
cGetValid        =     $01 
cGetSats         =     $02 
cGetTime         =     $03 
cGetDate         =     $04 
cGetLat          =     $05 
cGetLong         =     $06 
cGetAlt          =     $07 
cGetSpeed        =     $08 
cGetHead         =     $09 

cMyAddress       =     g#navBoard

'Sharp Dry Run Test Coords
'
'          Latitude        Longitude
'         ----------      -----------
' 0       29 40.4946      095 30.8299
' 1       29 40.4775      095 30.8540
' 2       29 40.4619      095 30.8795
' 3       29 40.4713      095 30.8883
' 4       29 40.5170      095 30.8784
' 5       29 40.5025      095 30.8934



OBJ
  G             :     "my_Globals"       
  f             :     "obex_Format"
  GPS           :     "GPS_IO_full"                     'allocates 1 cog
  GPSu          :     "GPSutilities"                    'allocates 2 cogs
  scanner       :     "Scanner"                         'allocates 2 cogs
  fs            :     "FloatString"
  fm            :     "FloatMath"
  rs            :     "my_RangerSerial"                    'allocates 1 cog
  'compass       :     "HM55B Compass Module Asm"        'allocates 1 cog


  
VAR
  long rc
  long packetFunc                                       'must be word or long so it can return both $00..$FF and -1

  long theTime[ 10 ]
  long theDate[ 10 ]
  
  byte map[ g#cScannerMaxRows+1 ]                         
  byte targLat[10],targLon[10]

  byte rxpacketBuf[g#maxData]
  byte txpacketBuf[g#maxData]
  byte arg1Buf[g#maxData]
  byte arg2Buf[g#maxData]
  byte arg3Buf[g#maxData]
  'byte arg4Buf[g#maxData]
  'byte arg5Buf[g#maxData]

  byte returnAddress,packetType
  byte myLock


  
PUB Main | cmd,dest,i,c
''test case: Distance between  29 42.8428'N 095 45.1912'W and  29 40.4946'N 095 30.8299'W is 14.6344 statute miles

' ---------------------------------------------------------------- 
' Init 
' ----------------------------------------------------------------
  myLock := lockNew


  GPS.start                                             'Start the GPS object
  GPSu.start
  fs.SetSeparatorChrs(",", 0)  

  bytefill(@map, 0, g#cScannerMaxRows)    
  'rc := compass.start(cEnablePin,cClockPin,cDataPin)
  rc := scanner.start(cServoPin,cSonarTrigger,cSonarData,@map)

  rs.start(cRxpin, cTxpin, %0000, g#cBaud,@rxpacketBuf,@txpacketBuf,cMyAddress,myLock)
  
  rs.tx(g#packetETX1)                                   'Cancel any unfinished transmission
  rs.tx(g#packetETX2)                                   'Cancel any unfinished transmission
  
  rs.str(string("REBOOT NAV"))                          'Cancel any unfinished transmission
  rs.tx(g#packetETX1)                                   'Cancel any unfinished transmission
  rs.tx(g#packetETX2)                                   'Cancel any unfinished transmission
  
' ---------------------------------------------------------------- 
' Init Complete -- Send a Ready Debug Packet
' ----------------------------------------------------------------

  rs.pkInit(g#comboard,g#commandPacket,g#debug)
  rs.pkAddStrArg(1,string("Ready",g#cr))
  rc := rs.pkSend

' ---------------------------------------------------------------- 
' Packet Process Loop  
' ---------------------------------------------------------------- 

  repeat
             waitcnt(cnt+(clkfreq*1))                          

    packetFunc := rs.pkReceiveCheck                       'did we get any data?

    if packetFunc == -1
      next      
          
    returnAddress := byte[@rxpacketBuf][g#offset_Source]
    packetType    := byte[@rxpacketBuf][g#offset_Type  ]
     
    case packetType
    
      g#commandPacket:             
        case packetFunc
          g#GetCogs:            sendFreeCogCount
          g#GetGPSinfo:         sendGPSinfo          
          g#getScanMap:         sendScanMap
          g#setWayPoint:        setWayPoint
          g#getDistAndBearing:  sendDistAndBearing
          g#doTest:             doTest
          
          other:
            f.sprintf(@arg1Buf,string("%d www is an invalid command function.nav"),packetFunc)  
            rs.pkInit(g#comboard,g#commandPacket,g#error)
            rs.pkAddStrArg(1,@arg1Buf)
            rc := rs.pkSend
       
      g#responsePacket:             
        case packetFunc
          g#GetCogs:               sendFreeCogCount

          other:
            f.sprintf(@arg1Buf,string("%d is an invalid response function.nav"),packetFunc)  
            rs.pkInit(g#comboard,g#commandPacket,g#error)
            rs.pkAddStrArg(1,@arg1Buf)
            rc := rs.pkSend
         
      g#acknowledgePacket:
        case packetFunc
          g#debug:                'debugMessage
          other:
            f.sprintf(@arg1Buf,string("%d is an invalid function acknowledge.nav"),packetFunc)  
            rs.pkInit(g#comboard,g#commandPacket,g#error)
            rs.pkAddStrArg(1,@arg1Buf)
            rc := rs.pkSend
                   
      other:
          waitcnt(cnt+(clkfreq*1))                          
            f.sprintf(@arg1Buf,string("%d www is an invalid packet type.nav"),packetType)  
            rs.pkInit(g#comboard,g#commandPacket,g#error)
            rs.pkAddStrArg(1,@arg1Buf)
            rc := rs.pkSend
         
    packetFunc := -1

pub error(strBuf) | L

        L := strsize(strBuf)
        rs.tx(g#packetSTX1)                 'Set Start-Of-Transmission 1
        rs.tx(g#packetSTX2)                 'Set Start-Of-Transmission 2
        rs.tx((g#offset_Data-g#offset_Start1)+L+2) 
        rs.tx(g#navBoard)
        rs.tx(g#comboard)
        rs.tx(g#error)
        rs.tx(g#commandPacket)
        rs.tx(1)    
        rs.tx("S")    
        rs.tx(L)    
        rs.str(strBuf)
        rs.tx(g#packetETX1)                 'Set End-Of-Transmission 1
        rs.tx(g#packetETX2)                 'Set End-Of-Transmission 2
                                


pub xbogus
    waitcnt(cnt+(clkfreq*1))                          
    rs.pkInit(g#comboard,g#commandPacket,g#debug)
    rs.pkAddStrArg(1,string("bogus="))
    rs.pkAddStrArg(2,g.lookupFunctionName(packetFunc))
    rc := rs.pkSend
    waitcnt(cnt+(clkfreq*1))                          


pub doTest  | i

    rs.pkInit(g#motBoard,g#responsePacket,g#debug)
    rs.pkAddStrArg(1,string("Hello"))
    rc := rs.pkSend




Pub setWayPoint   | n,c,ptr

 

   
    
Pub sendFreeCogCount | n 

  rs.pkInit(returnAddress,g#responsePacket,packetFunc)
  n := g.free_cogs
  rs.pkAddNumArg(1,n)
  rc := rs.pkSend






       
pub sendDistAndBearing | latRads1,lonrads1,latRads2,lonrads2,distance,bearing
''test case: Distance between  29 42.8428'N 095 45.1912'W and  29 40.4946'N 095 30.8299'W is 14.6344 statute miles

    latRads1 := GPSu.convert2Radians(GPS.N_S,GPS.latitude)            'returns floating point radians
    lonRads1 := GPSu.convert2Radians(GPS.E_W,GPS.longitude)           'returns floating point radians

    latRads2 := GPSu.convert2Radians(string("N"),targLat)             'returns floating point radians
    lonRads2 := GPSu.convert2Radians(string("W"),targLon)             'returns floating point radians

    distance := GPSu.calcHaversineDistance(latRads1,lonRads1,latRads2,lonRads2) 
    bearing  := GPSu.calculateBearing(latRads1,lonRads1,latRads2,lonRads2,distance) 

    rs.pkInit(returnAddress,g#responsePacket,g#getDistAndBearing)
    'rs.pkAddNumArg(fm.FRound(distance))
    'rs.pkAddNumArg(fm.FRound(bearing))
    rc := rs.pkSend
    'waitcnt(cnt+(clkfreq/1000))                         

        
pub sendGPSinfo | sats,n,i,c,latRads1,lonrads1,latRads2,lonrads2
         
    i := f.atoi(GPS.FixQuality)  
    
    GPS.localTime(@theDate,@theTime)

    rs.pkInit(g#comboard,g#responsePacket,g#GetGPSinfo)
    
    rs.pkAddStrArg(g#gpsDate                  ,theDate)
    rs.pkAddStrArg(g#gpsTime                  ,theTime)
    rs.pkAddStrArg(g#gpsHeadingDegreesStr     ,GPS.headingDegreesStr)
    n:=GPS.satelliteCount                                                        
    rs.pkAddNumArg(g#gpsSatelliteCount        ,n)
    rs.pkAddNumArg(g#gpsAltitudeFeet          ,n:=GPS.altitudeFeet)
    rs.pkAddNumArg(g#gpsSpeedInMph            ,n:=GPS.speedInMph)
    rs.pkAddNumArg(g#gpsFixQuality            ,n:=GPS.fixQuality)
    
    rs.pkAddStrArg(g#gpsLatitude              ,GPS.latitude)
    rs.pkAddStrArg(g#gpsN_S                   ,GPS.N_S)
    rs.pkAddStrArg(g#gpsLongitude             ,GPS.longitude)
    rs.pkAddStrArg(g#gpsE_W                   ,GPS.E_W)
    rs.pkAddStrArg(g#gpsHeadingCompassPoint   ,GPS.headingCompassPoint)
    
    rc := rs.pkSend
    
   'waitcnt(cnt+(clkfreq*3))                          
     



pub sendScanMap | j,n,c,bc
    'bc := calcBestCourse
    'g.Bin2Hex(@arg1Buf,@map,g#cScannerMaxRows)

    rs.pkInit(g#comboard,g#responsePacket,g#getScanMap)
    rs.pkAddHexArg(1,@map,g#cScannerMaxRows)
    rc := rs.pkSend


            
pub calcBestCourse | j,ncurrent,nbest,best

  best := 0
  repeat j from 1 to g#cScannerMaxRows-1
  
    ncurrent := (map[ j-1 ]    + map[ j ]    + map[ j+1 ]   ) / 3
    nbest    := (map[ best-1 ] + map[ best ] + map[ best+1 ]) / 3

    if ncurrent > nbest
      best := j

  return best



  