'' *****************************
''
'' NMEA0183 GPS routines
''
'' *****************************
CON

   CR = 13                              ' ASCII <CR>
   LF = 10                              ' ASCII <LF>

   serRecv   = 8                      ' Serial Receive
   
VAR 
   long gps_stack[50]
    
   byte GP_RMC_b  [90], GP_GGA_b [90], GP_GSA_b [90]   
   long GP_RMC_a  [20], GP_GGA_a [20], GP_GSA_a [20]
   
   byte GP_GSV_1b [90], GP_GSV_2b[90], GP_GSV_3b[90]   
   long GP_GSV_1a [20], GP_GSV_2a[20], GP_GSV_3a[20]
   
   byte gps_buff[90],Rx,chk_xsum,chk_xval
   long cog,cptr,ptr,arg,j
   long Null[1]
   long rmc_ready
     

OBJ
  G             : "my_Globals"       
  uart          : "obex_HITT_Serial_Recv_Buffer_True_32"
  format        : "obex_Format"       
  'serial        : "MultiCogSerial" 
   Cal          : "obex_Calendar"

PUB start : okay

'' Starts uart object (at baud specified) in a cog
'' -- returns false if no cog available

'   okay := uart.start(14,4800) ' (serRecv,serXmit,0,4800)
    okay := uart.Init (serRecv,4800)
    return cog := cognew(readNEMA,@gps_stack) 

pri readNEMA

Null[0] := rmc_ready := 0

repeat
  'longfill(gps_buff,0,20)

  repeat while Rx <> "$"        ' wait for the $ to insure we are starting with
     Rx := uart.Rx              '   a complete NMEA sentence 

  cptr     := 0
  chk_xsum := 0
  
  repeat while Rx <> "*"        '  continue to collect data until the end of the NMEA sentence 
     Rx := uart.Rx
                  
     if Rx == ","
       gps_buff[cptr++] := 0    '  Replace the character
     else
       gps_buff[cptr++] := Rx   '  Save the character 

     if cptr > 82
       quit

     if Rx <> "*"
      chk_xsum ^= Rx                                                                                                       

  cptr--
      
  chk_xval := to_hex(uart.Rx) << 4
  chk_xval += to_hex(uart.Rx)
  
  if chk_xsum == chk_xval
    
   if gps_buff[2] == "R"
     if gps_buff[3] == "M"
       if gps_buff[4] == "C"
          rmc_ready := 0
          copy_buffer(@GP_RMC_b, @GP_RMC_a)
                
   if gps_buff[2] == "G"
     if gps_buff[3] == "G"  
       if gps_buff[4] == "A"
           copy_buffer(@GP_GGA_b, @GP_GGA_a)

   if gps_buff[2] == "G"
     if gps_buff[3] == "S"
       if gps_buff[4] == "A"
           copy_buffer(@GP_GSA_b, @GP_GSA_a)
           
   if gps_buff[2] == "G"
     if gps_buff[3] == "S"
       if gps_buff[4] == "V"       
         if gps_buff[8] == "1"
               copy_buffer(@GP_GSV_1b, @GP_GSV_1a)               
         if gps_buff[8] == "2"
               copy_buffer(@GP_GSV_2b, @GP_GSV_2a)               
         if gps_buff[8] == "3"
               copy_buffer(@GP_GSV_3b, @GP_GSV_3a)
       rmc_ready := 1
               
pri copy_buffer ( buffer,args)
         bytemove(buffer,@gps_buff,cptr)
         ptr := buffer
         arg := 0
         repeat j from 0 to 82
          if arg > 19
            quit
 
          if byte[ptr] == 0
             long[args][arg++] := ptr+1
          ptr++
          

pri to_hex(key)

  case key
   "0": return 0
   "1": return 1
   "2": return 2
   "3": return 3 
   "4": return 4
   "5": return 5
   "6": return 6
   "7": return 7
   "8": return 8
   "9": return 9
   "A": return 10
   "B": return 11
   "C": return 12
   "D": return 13
   "E": return 14
   "F": return 15
             
pri xsum
   return chk_xsum                                                                                               
   
pri xval
   return chk_xval
   
pub altitudeMetersStr

'' Altitude, Meters, above mean sea level

   return GP_GGA_a[8]
   
pub altitudeFeet | feet

'' Altitude, Meters, above mean sea level
   feet := (Format.atoi(GP_GGA_a[8])* 32808)/10000
   return feet
   
pub speedKnotsStr

'' Speed over the ground in knots

   return GP_RMC_a[6]

pub speedInMph | mph

'' Speed over the ground in MPH (returns a number)
   mph := (format.atoi(GP_RMC_a[6])*1152)/1000
   return mph 

pub headingDegreesStr

'' Track angle in degrees True

   return GP_RMC_a[7]

pub headingCompassPoint | h,s

'' Track angle in degrees True

   h := format.atoi(GP_RMC_a[7])
   case h
      000..022: s := String("N ") 
      023..067: s := String("NE")
      068..112: s := String("E ")
      113..157: s := String("SE")
      158..202: s := String("S ")
      203..247: s := String("SW")
      247..292: s := String("W ")
      293..337: s := String("NW")
      338..360: s := String("N ")
   return s

pub timehh  | hh

    hh := to_hex(byte[GP_GGA_a[0]][0]) *10   'uses existing hex lookup as a decimal lookup
    hh += to_hex(byte[GP_GGA_a[0]][1])
    return hh
    
pub timemm  | mm
    mm := to_hex(byte[GP_GGA_a[0]][2]) *10   'uses existing hex lookup as a decimal lookup
    mm += to_hex(byte[GP_GGA_a[0]][3])
    return mm

pub timess  | ss
    ss := to_hex(byte[GP_GGA_a[0]][4]) *10   'uses existing hex lookup as a decimal lookup
    ss += to_hex(byte[GP_GGA_a[0]][5])
    return ss



pub localtime(dateStr,timeStr) | hh,mm,ss, month,day,yyyy

  hh := timehh
  mm := timemm
  ss := timess
  
  month := dateMonth
  day   := dateDay
  yyyy  := dateYear

  Cal.ConvertUTCtoLocaltime( -6, yyyy, month, day, hh, mm, ss, true, dateStr, timeStr)  

   
pub timeHHMMSS

'' UTC time of fix HHMMSS

   return GP_GGA_a[0]
   
pub dateDay | dd
    dd := to_hex(byte[GP_RMC_a[8]][0]) *10   'uses existing hex lookup as a decimal lookup
    dd += to_hex(byte[GP_RMC_a[8]][1])
    return dd

pub dateMonth  | mm
    mm := to_hex(byte[GP_RMC_a[8]][2]) *10   'uses existing hex lookup as a decimal lookup
    mm += to_hex(byte[GP_RMC_a[8]][3])
    return mm

pub dateYear  | yy
    yy := to_hex(byte[GP_RMC_a[8]][4]) *10   'uses existing hex lookup as a decimal lookup
    yy += to_hex(byte[GP_RMC_a[8]][5])
    return yy + 2000

pub dateDDMMYY

'' DDMMYY

   return GP_RMC_a[8]
    

pub latitudeRadians

'' degrees and minutes - dddmm.mmmm 
   return GP_GGA_a[1]
    
pub latitudeDegrees

'' degrees and minutes - dddmm.mmmm 
   return GP_GGA_a[1]
    
pub latitude

'' degrees and minutes - dddmm.mmmm 
   return GP_GGA_a[1]
    
pub N_S

'' N = north or S = south 

   return GP_GGA_a[2]
     
pub longitude

'' degrees and minutes - dddmm.mmmm 

   return GP_GGA_a[3]
    
pub E_W

'' E = east or W = west 

   return GP_GGA_a[4]

pub FixQuality

{{ 
    0 = invalid
    1 = GPS fix (SPS)
    2 = DGPS fix
    3 = PPS fix
    4 = Real Time Kinematic
    5 = Float RTK
    6 = estimated (dead reckoning) (2.3 feature)
    7 = Manual input mode
    8 = Simulation mode
}}
   return GP_GGA_a[5]

pub satellites

'' Number of satellites being tracked (returns String)

   return GP_GGA_a[6]
  
pub satelliteCount | n

'' Number of satellites being tracked (returns a number)
     
    n := to_hex(byte[GP_GGA_a[6]][0]) *10   'uses existing hex lookup as a decimal lookup
    n += to_hex(byte[GP_GGA_a[6]][1])

   return n
  
pub hdop
   return GP_GGA_a[7]
   
pub vdop
   return GP_GSA_a[14]

pub ready
   return rmc_ready

pub sat_id (sat)

'' Ranges from 1 to 32

    case sat
      1:
          return GP_GSV_1a[3]
      2:
          return GP_GSV_1a[7]
      3:
          return GP_GSV_1a[11]
      4:
          return GP_GSV_1a[15]
      5:
          return GP_GSV_2a[3]
      6:
          return GP_GSV_2a[7]
      7:
          return GP_GSV_2a[11]
      8:
          return GP_GSV_2a[15]
      9:
          return GP_GSV_3a[3]
      10:
          return GP_GSV_3a[7]
      11:
          return GP_GSV_3a[11]
      12:
          return GP_GSV_3a[15]
      other:
          return 0
          
pub sat_elevation (sat)

'' Ranges from 0 to 90

    case sat
      1:
          return GP_GSV_1a[4]
      2:
          return GP_GSV_1a[8]
      3:
          return GP_GSV_1a[12]
      4:
          return GP_GSV_1a[16]
      5:
          return GP_GSV_2a[4]
      6:
          return GP_GSV_2a[8]
      7:
          return GP_GSV_2a[12]
      8:
          return GP_GSV_2a[16]
      9:
          return GP_GSV_3a[4]
      10:
          return GP_GSV_3a[8]
      11:
          return GP_GSV_3a[12]
      12:
          return GP_GSV_3a[16]
      other:
          return 0
          
pub sat_azimuth (sat)

'' True, Range 0 to 359

    case sat
      1:
          return GP_GSV_1a[5]
      2:
          return GP_GSV_1a[9]
      3:
          return GP_GSV_1a[13]
      4:
          return GP_GSV_1a[17]
      5:
          return GP_GSV_2a[5]
      6:
          return GP_GSV_2a[9]
      7:
          return GP_GSV_2a[13]
      8:
          return GP_GSV_2a[17]
      9:
          return GP_GSV_3a[5]
      10:
          return GP_GSV_3a[9]
      11:
          return GP_GSV_3a[13]
      12:
          return GP_GSV_3a[17]
      other:
          return 0         

pub sat_snr (sat)

'' Range 0 to 99, null when not tracking

    case sat
      1:
          return GP_GSV_1a[6]
      2:
          return GP_GSV_1a[10]
      3:
          return GP_GSV_1a[14]
      4:
          return GP_GSV_1a[18]
      5:
          return GP_GSV_2a[6]
      6:
          return GP_GSV_2a[10]
      7:
          return GP_GSV_2a[14]
      8:
          return GP_GSV_2a[18]
      9:
          return GP_GSV_3a[6]
      10:
          return GP_GSV_3a[10]
      11:
          return GP_GSV_3a[14]
      12:
          return GP_GSV_3a[18]
      other:
          return 0
                            